@@ -29,6 +29,8 @@
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreFoundation/CFMachPort.h>
 
+#include <libkern/OSCrossEndian.h>
+
 #include <IOKit/IOBSD.h>
 #include <IOKit/IOKitLib.h>
 #include <IOKit/IOKitServer.h>
@@ -803,9 +805,8 @@ IODispatchCalloutFromCFMessage(CFMachPortRef port, void *_msg, CFIndex size, voi
     // If this is ppc code running under Rosetta over an x86 kernel,
     // we may need to do some byte swapping. The magic cookie in the
     // inline asm makes Rosetta ignore this branch command.
-    asm volatile ("b L_SkipSwap_label \n .long 0x14400004");
-    swapOSNotification(header, leftOver);
-    asm volatile ("L_SkipSwap_label:");
+    IF_ROSETTA()
+	swapOSNotification(header, leftOver);
 #endif
 
     // remote port is the notification (an iterator_t) that fired
@@ -821,6 +822,7 @@ IODispatchCalloutFromCFMessage(CFMachPortRef port, void *_msg, CFIndex size, voi
 	    deliver = false;
 	}
     }
+
     if(deliver)
     {
       switch( header->type )
@@ -1000,8 +1002,24 @@ IOServiceOpen(
 	unsigned int	type,
 	io_connect_t  *	connect )
 {
-    return( io_service_open( service,
-	owningTask, type, connect ));
+    kern_return_t	kr;
+    kern_return_t	result;
+
+    kr = io_service_open_extended( service,
+	owningTask, type, NDR_record, NULL, 0, &result, connect );
+
+#if 1
+    if (MIG_BAD_ID == kr)
+    {
+	kr = io_service_open(service, owningTask, type, connect);
+	result = kr;
+    }
+#endif
+
+    if (KERN_SUCCESS == kr)
+        kr = result;
+
+    return (kr);
 }
 
 kern_return_t
@@ -1595,6 +1613,123 @@ IORegistryEntryGetProperty(
 	io_struct_inband_t	buffer,
 	unsigned int	      * size )
 {
+
+#if __ppc__
+    // If this is ppc code running under Rosetta over an x86 kernel,
+    // we may need to do some byte swapping. The magic cookie in the
+    // inline asm makes Rosetta ignore this branch command.
+    IF_ROSETTA()
+    {
+	IOReturn ret = kIOReturnSuccess;
+	do
+	{
+	    CFTypeID	 type;
+	    CFTypeRef    obj;
+	    CFStringRef  key;
+	    CFDataRef    dataBuffer = 0;
+	    const void * bytes = 0;
+	    size_t       len = 0;
+	    bool	 addNull = false;
+	    
+	    key = CFStringCreateWithCString (kCFAllocatorDefault, name, kCFStringEncodingUTF8);
+	    if (!key)
+	    {
+		ret = kIOReturnNoMemory;
+		break;
+	    }
+	    obj = IORegistryEntryCreateCFProperty(entry, key, kCFAllocatorDefault, kNilOptions);
+	    CFRelease(key);
+	    if (!obj)
+	    {
+		ret = kIOReturnNoResources;
+		break;
+	    }
+
+	    type = CFGetTypeID(obj);
+
+	    if (type == CFDataGetTypeID())
+	    {
+		len = CFDataGetLength(obj);
+		bytes = CFDataGetBytePtr(obj);
+	    }
+	    else if (type == CFStringGetTypeID())
+	    {
+		dataBuffer = CFStringCreateExternalRepresentation(kCFAllocatorDefault, obj, kCFStringEncodingUTF8, 0);	
+		if (!dataBuffer)
+		    dataBuffer = CFStringCreateExternalRepresentation(kCFAllocatorDefault, obj, kCFStringEncodingUTF8, (UInt8)'?');
+		if (dataBuffer)
+		{
+		    len = CFDataGetLength(dataBuffer);
+		    bytes = CFDataGetBytePtr(dataBuffer);
+		    addNull = true;
+		}
+		else
+		    ret = kIOReturnNoResources;
+	    }
+	    else if (type == CFBooleanGetTypeID())
+	    {
+		if (CFBooleanGetValue(obj))
+		{
+		    len   = sizeof("Yes");
+		    bytes = "Yes";
+		}
+		else
+		{
+		    len   = sizeof("No");
+		    bytes = "No";
+		}
+	    }
+	    else if (type == CFNumberGetTypeID())
+	    {
+		CFNumberType numType;
+		switch (*size)
+		{
+		    case 1:
+			numType = kCFNumberSInt8Type;
+			break;
+		    case 2:
+			numType = kCFNumberSInt16Type;
+			break;
+		    case 4:
+			numType = kCFNumberSInt32Type;
+			break;
+		    case 8:
+			numType = kCFNumberSInt64Type;
+			break;
+		    default:
+			ret = kIOReturnBadArgument;
+			break;
+		}
+		if (kIOReturnSuccess == ret)
+		{
+		    if (!CFNumberGetValue(obj, numType, buffer))
+			ret = kIOReturnNoResources;
+		}
+	    }
+	    else
+		ret = kIOReturnBadArgument;
+
+	    if (bytes)
+	    {
+		if (*size < (len + addNull))
+		    ret = kIOReturnIPCError;
+		else
+		{
+		    bcopy(bytes, buffer, len);
+		    if (addNull)
+			buffer[len++] = 0;
+		    *size = len;
+		}
+	    }
+	    if (dataBuffer)
+		CFRelease(dataBuffer);
+	    CFRelease(obj);
+	}
+	while (false);
+	return (ret);
+    }
+#endif
+
     return( io_registry_entry_get_property_bytes( entry, (char *) name,
 						  buffer, size ));
 }
